#-*- coding: utf-8 -*-
#
#       Copyright (C) 2010 the sch-scripts developers
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import pygtk
pygtk.require('2.0')
import gtk
from gobject import TYPE_PYOBJECT as gPyobject
import gobject
import os
import re
import dbus
import logging
import sys
import shlex
from schscripts.client.notify import *
from schscripts.client.executecommand import *
from schscripts.client.create_users import CreateUsersDialog
import schscripts.client.ltsconf as ltsConf
import schscripts.client.wol as wol
ltsConf = ltsConf.ltsConf()

from twisted.internet import reactor
from twisted.python import log

from schscripts.client import connection
from schscripts.lib_users import *

# Constants for the client iconview, e.g.
# ['ltsp123', '00-1b-24-89-65-d6', '127.0.0.1:46827', '10.160.31.126:44920', 'thin', 'user3', <gtk.gdk.Pixbuf>, '10.160.31.123']
C_HOSTNAME = 0
C_MAC = 1
C_SESSION_HANDLE = 2
C_SYSTEM_HANDLE = 3
C_TYPE = 4
C_USER = 5
C_PIXBUF = 6
C_IP = 7

# Warn users to update their chroots if they have a lower sch-client version than this
COMPATIBILITY_VERSION = [10, 4, 34]

class SchGui(object):

    def __init__(self, conf):
        self.conf = conf
        
        # Enable loggings
        #logging.basicConfig(filename='/var/log/sch-scripts.log', level=logging.INFO)
        self.shownCompatibilityWarning = False 
        self.added = False
        self.vncserver_running = False
        self.scrWidth = 100
        self.scrHeight = 75
        self.offline = gtk.gdk.pixbuf_new_from_file('images/offline.svg')
        self.thin = gtk.gdk.pixbuf_new_from_file('images/thin.svg')
        self.fat = gtk.gdk.pixbuf_new_from_file('images/fat.svg')
        self.standalone = gtk.gdk.pixbuf_new_from_file('images/standalone.svg')
        self.imagetypes = {'thin' : self.thin, 'fat' : self.fat, 'standalone' : self.standalone}
        self.addStockImage('offline', 'images/off.png')
        self.addStockImage('online', 'images/on.png')
        self.addStockImage('users', 'images/usersgrp.png')
        self.addStockImage('system', 'images/systemgrp.png')
        
        #self.conf = ConfigParser.ConfigParser()
        #self.conf.read('sch-scripts.conf')

        # Get DBUS_SESSION_BUS_ADDRESS from the parent
        parentenvf = open("/proc/%d/environ" % os.getppid())
        parentenv = parentenvf.read().split('\0')
        parentenvf.close()
        for env in parentenv:
            if env.startswith('DBUS_SESSION_BUS_ADDRESS='):
                self.dbus_session_bus_address = env.lstrip('BUS_SESSION_BUS_ADDRESS=')
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('sch-scripts.glade')
        
        # Connect glade handlers with the callback functions
        self.wTree.connect_signals(self)
        self.get = lambda obj: self.wTree.get_object(obj)
        
        self.get('iconsSizeAdjustment').set_value(self.scrWidth)
        
        self.mainwin = self.get('mainwindow')
        
        self.sysgroups = self.get('sysgroups').get_active()
        self.emptygroups = self.get('emptygroups').get_active()
        
        #self.applyConf()
        #self.mainwin.parse_geometry(self.conf.get('General', 'geometry'))
        
        #[3] is status (replace with gdk.Pixbuf)
        self.ustore = gtk.ListStore(str, str, str, str, str, gPyobject)
        self.gstore = gtk.ListStore(str, str, gPyobject, str)
        #self.cstore = gtk.ListStore(str, str, str, str, str, gtk.gdk.Pixbuf)
        self.cstore = gtk.ListStore(str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
        #self.loadGroups()
        
        
        self.utree = self.get('userstree')
        self.gtree = self.get('groupstree')
        self.cview = self.get('clientsview')
        self.refresh()
        self.utree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.gtree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        
        self.ufilter = self.ustore.filter_new()
        self.gfilter = self.gstore.filter_new()
        self.cfilter = self.cstore.filter_new()
        self.ufilter.set_visible_func(self.setVisibleUsers)
        self.gfilter.set_visible_func(self.setVisibleGroups)
        self.cfilter.set_visible_func(self.setVisibleClients)
        
        self.csort = gtk.TreeModelSort(self.cfilter)
        
        self.utree.set_model(self.ufilter)
        self.gtree.set_model(self.gfilter)
        self.cview.set_model(self.csort)
        
        # Add the columns for the users tree
        self.utree.columns = [None]*3
        self.utree.columns[0] = gtk.TreeViewColumn('', gtk.CellRendererPixbuf(), stock_id=3)
        self.utree.columns[1] = gtk.TreeViewColumn('Χρήστες', gtk.CellRendererText(), text=0)
        self.utree.columns[2] = gtk.TreeViewColumn('Χρήστες', gtk.CellRendererText(), text=2)
        self.utree.columns[0].set_reorderable(True)
        self.utree.columns[0].set_sort_column_id(0)
        self.utree.append_column(self.utree.columns[0])
        for i in range(1,3):
            self.utree.columns[i].set_resizable(True)
            self.utree.columns[i].set_reorderable(True)
            #self.utree.columns[i].set_sort_column_id(i)
            self.utree.append_column(self.utree.columns[i])
            
        # Add the columns for the groups tree
        self.gtree.columns = [None]*2
        self.gtree.columns[0] = gtk.TreeViewColumn('', gtk.CellRendererPixbuf(), stock_id=3)
        self.gtree.columns[1] = gtk.TreeViewColumn('Ομάδες', gtk.CellRendererText(), text=0)
        for i in range(len(self.gtree.columns)):
            self.gtree.columns[i].set_resizable(True)
            self.gtree.columns[i].set_reorderable(True)
            #self.gtree.columns[i].set_sort_column_id(i)
            self.gtree.append_column(self.gtree.columns[i])
        
        self.cview.set_text_column(0)
        self.cview.set_pixbuf_column(6)
        #self.refresh()
        self.csort.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self.gfilter.refilter()
        self.setPropertiesSensitivity()
        self.toggleRealNameColumn()
        self.createListsView()

    
    def connected(self, daemon):
        self.daemon = daemon
        daemon.enumerateClients().addCallback(lambda h: self.amp_gotClients(h))
        #gobject.timeout_add(5000, self.getScreenshots)

    # AMP callbacks

    def amp_clientConnected(self, handle):
        print handle, "connected. ",
        self.daemon.command(handle, u"""
            VERSION=$(dpkg-query -W -f '${Version}' sch-client 2>/dev/null)
            echo "$USER\n$HOSTNAME\n$IP\n$MAC\n$TYPE\n$UID\n$VERSION"
            """).addCallback(lambda r: self.addClient(handle, r))

    def amp_clientDisconnected(self, handle):
        for client in self.cstore:
            if client[C_SYSTEM_HANDLE] == handle:
                shutdownNotify(client[C_HOSTNAME])
                if client[C_SESSION_HANDLE] == '':
                    if ltsConf.sectionExists(client[C_MAC]):
                        self.savedClientReset(client)
                    else:
                        self.cstore.remove(client.iter)
                else:
                    client[C_SYSTEM_HANDLE] = ''
                break
            elif client[C_SESSION_HANDLE] == handle:
                logoutNotify(client[C_USER], client[C_HOSTNAME])
                if client[C_SYSTEM_HANDLE] == '':
                    if ltsConf.sectionExists(client[C_MAC]):
                        self.savedClientReset(client)
                    else:
                        self.cstore.remove(client.iter)            
                else:
                    client[C_USER] = ''
                    client[C_SESSION_HANDLE] = ''
                    type = client[C_TYPE]
                    client[C_PIXBUF] = self.imagetypes[type]
                break
        self.refresh()
        print handle, "disconnected. Remaining client list: "
        for client in self.cstore:
            print [ value for value in client ]
    
    def savedClientReset(self, client):
        client[C_SESSION_HANDLE] = ''
        client[C_SYSTEM_HANDLE] = ''
        client[C_USER] = ''
        client[C_TYPE] = 'offline'
        client[C_PIXBUF] = self.offline

    def amp_gotClients(self, handles):
        print "Existing client list:"
        for handle in handles:
            print "   ", handle
            d = self.daemon.command(handle,  u"""
                VERSION=$(dpkg-query -W -f '${Version}' sch-client 2>/dev/null)
                echo "$USER\n$HOSTNAME\n$IP\n$MAC\n$TYPE\n$UID\n$VERSION"
                """)
            d.addCallback(lambda r, h=handle: self.addClient(h, r, True))
        #self.getScreenshots()
    
    #TODO: Move this def 
    def on_button_close_clicked(self, widget):
        self.get('warningDialog').hide()
    
    def addClient(self, handle, r, already=False):
        # already is True if the client was started before sch-scripts
        user, host, ip, mac, type, uid, version = r.strip().split()
        
        if [int(s) for s in version.split('.')] < COMPATIBILITY_VERSION:
            
            if not self.shownCompatibilityWarning:
                self.shownCompatibilityWarning = True
                dlg = self.get('warningDialog')
                msg = "Έγινε προσπάθεια σύνδεσης ενός sch-client με έκδοση %s η οποία είναι ασύμβατη με την τρέχουσα έκδοση των sch-scripts.\n\nΘα πρέπει να ενημερώσετε τον εικονικό δίσκο σας πηγαίνοντας στο μενού Εξυπηρετητής → Εικονικός δίσκος → Ενημέρωση, και στη συνέχεια Συμπίεση." % version
                dlg.set_property('text', msg)
                dlg.set_transient_for(self.mainwin)
                dlg.show()
            self.daemon.command(handle, u"exit")
            return
            
        mac = mac.upper()
        index = None
        for i in range(len(self.cstore)):
            if self.cstore[i][C_IP] == ip or (self.cstore[i][C_MAC] == mac and self.cstore[i][C_IP] == '') \
              or (self.cstore[i][C_HOSTNAME] == host and self.cstore[i][C_IP] == '' and type == 'thin' and uid != 0):
                index = i
                entry = self.cstore[i]
                break

        if index == None:
            entry = ['', '', '', '', '', '', '', '']

        if int(uid) == 0:
            entry = [host, mac, entry[C_SESSION_HANDLE], handle, type, entry[C_USER], self.imagetypes[type], ip]
        else:
            # For standalone clients, use mac etc if not provided already
            entry = [host, entry[C_MAC], handle, entry[C_SYSTEM_HANDLE], type, user, self.imagetypes[type], ip]
            gobject.timeout_add(5000, self.getScreenshots, entry[C_SESSION_HANDLE])
            if already:
                loggedinNotify(user, host)
            else:
                loginNotify(user, host)
        if index != None:
            self.cstore[index] = entry
        else:
            self.cstore.append(entry)
        
        self.refresh()
        print "New client list:"
        for client in self.cstore:
            print [ value for value in client ]
    
    def getAllScreenshots(self):
        pass
        # This should ask for screenshots for every client but makes
        # twisted throw an error.
        #for client in self.cstore:
        #    self.getScreenshots(client[C_SESSION_HANDLE])
        #    #self.execOnClients('./sch-screenshot %i %i || exit 0' %(self.scrWidth, self.scrHeight), 
        #    #handles=[client[C_SESSION_HANDLE]], reply=self.updateScreenshots)
    
    
    def getScreenshots(self, handle):
        # If the client can't get a screenshot, it's probably logged off,
        # so exit sch-client.
        exists = False
        for client in self.cstore:
            if handle == client[C_SESSION_HANDLE]:
                exists = True
                break
        if exists:
            self.execOnClients('if ./sch-screenshot %i %i; then BAD_SCREENSHOTS=0; elif [ "$BAD_SCREENSHOTS" = 3 ]; then exit 1; else BAD_SCREENSHOTS=$(($BAD_SCREENSHOTS+1)); fi'
                % (self.scrWidth, self.scrHeight), handles=[handle], reply=self.updateScreenshots)
            return True
        
        return False
        
    
    def updateScreenshots(self, handle, reply):
        if reply == None:
            return
        try:
            rowstride, size, pixels = reply.split('\n', 2)
        except:
            return
        rowstride = int(rowstride)
        width, height = size.split('x')
        for i in self.cstore:
            if handle == i[C_SESSION_HANDLE]:
                pxb = gtk.gdk.pixbuf_new_from_data(pixels, gtk.gdk.COLORSPACE_RGB, 
                                            True, 8, int(width), int(height), rowstride)
                self.cstore[i.path][C_PIXBUF] = pxb
                break
    
    def applyConf(self):
        if self.conf.getboolean('General', 'maximized'):
            self.mainwin.maximize()
        
    def saveConf(self, item, value):
        self.conf.set('General', item, str(value))
    
    def addStockImage(self, name, filename):
        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)#_at_size(filename,16,16)
        iconset = gtk.IconSet(pixbuf)
        
        factory = gtk.IconFactory()
        factory.add(name, iconset)
        factory.add_default()
    
    def loadUsers(self):
        """Puts the users in the gtkTreeView"""
        users = read_users_from_passwd()
        self.ustore.clear()
        bus = dbus.SystemBus()
        ck = bus.get_object('org.freedesktop.ConsoleKit', 
                                  '/org/freedesktop/ConsoleKit/Manager')
        GetSessionsForUnixUser = ck.get_dbus_method(
         'GetSessionsForUnixUser', 'org.freedesktop.ConsoleKit.Manager')
        
        for name, u in users.iteritems():
            sessions=GetSessionsForUnixUser(u.uid)
            if len(sessions) == 0:
                status='offline'
            else:
                status='online'
            #self.ustore.append([status, u.rname, u.name])
            self.ustore.append([u.name, u.uid, u.rname, status, u.dir, \
                      u.groups])
        #name, uid='', rname='', office='', wphone='',
        #hphone='', dir='', shell='', groups=[], min='', max='',
        #warn='', inact='', expire='', pwd='', plainpw=''
    
    def loadClients(self):
        macs = ltsConf.getSavedClients()#Sections()
        for mac in macs:
            mac = mac.upper()
            exists = False
            for client in self.cstore:
                if client[C_MAC] == mac:
                    exists = True
                    #hostname = ltsConf.getItem(mac, 'HOSTNAME')
                    #client[C_HOSTNAME] == hostname
                    break
            if not exists:
                hostname = ltsConf.getItem(mac, 'HOSTNAME')
                self.cstore.append([hostname, mac, '', '', 'offline', '', self.offline, ''])
    
    def loadGroups(self):
        self.gstore.clear()
        grps = grp.getgrall()
        for g in grps:
            type = 'users'
            if g.gr_gid < 1000 or g.gr_gid >= 60000:
                type='system'
            
            self.gstore.append([g.gr_name, g.gr_gid, g.gr_mem, type])
    
    def getSelectedClients(self):
        selected = self.cview.get_selected_items()
        items = []
        for i in selected:
            path = self.cfilter.convert_path_to_child_path(self.csort.convert_path_to_child_path(i[0]))
            items.append(self.cstore[path])
        return items
    
    def getSelectedUsers(self):
        selected = self.utree.get_selection().get_selected_rows()[1]
        items = []
        for i in selected:
            path = self.ufilter.convert_path_to_child_path(i[0])
            items.append(self.ustore[path])
        return items
    
    def getSelectedGroups(self): 
        selected = self.gtree.get_selection().get_selected_rows()[1]
        items = []
        for i in selected:
            path = self.gfilter.convert_path_to_child_path(i[0])
            items.append(self.gstore[path])
        return items
    
    def setVisibleUsers(self, model, iter):
        return True #Show every user
        
    def setVisibleClients(self, model, iter):
        return True
    
    def setVisibleGroups(self, model, iter):
        #print model.get_value(iter, 0)
        memb = model[iter][2]
        gtype = model.get_value(iter, 3)
        
        if self.sysgroups == True and self.emptygroups == True:
            return True
            
        elif self.sysgroups == True and self.emptygroups == False:
            if memb:
                return True
        
        elif self.sysgroups == False and self.emptygroups == True:
            if gtype != 'system':
                return True

        elif self.sysgroups == False and self.emptygroups == False:
            if gtype != 'system' and memb:
                #print model[iter][0], memb
                return True
        return False
        
    def changeHostname(self, mac, new_name):
        pass
    
    
    
    
    # Callback functions
    def refresh(self, widget=None):
        ltsConf.parse()
        self.loadClients()
        self.loadGroups()
        #self.gfilter.refilter()
        self.loadUsers()
        self.setPropertiesSensitivity()
        
    def on_mainwin_destroy(self, widget):
        reactor.stop()
  
    def aboutDialog(self, widget):
        version = '10.04' #FIXME: Get the version automatically
        dialog = self.get('aboutdialog')
        logo = gtk.gdk.pixbuf_new_from_file_at_size('../pixmaps/sch-scripts.svg', 48, 48)
        dialog.set_logo(logo)
        dialog.set_version(version)
        
        dialog.run()
        dialog.hide()
    
    def importExportUsers(self, widget=None):
        subprocess.Popen(['scripts/users/import-export-users'])

    def createUsersDlg(self, widget=None):
        CreateUsersDialog()

    def toggleRealNameColumn(self, widget=None):
        if self.get('mcShowRealName').get_active():
            self.utree.remove_column(self.utree.columns[1])#.hide()
            if not self.utree.columns[2] in self.utree.get_columns():
                self.utree.append_column(self.utree.columns[2])
        else:
            self.utree.remove_column(self.utree.columns[2])
            if not self.utree.columns[1] in self.utree.get_columns():
                self.utree.append_column(self.utree.columns[1])
    
    def installListsOnServer(self, lists):
        pars = ' '.join(lists)
        self.execInTerminal(self.get('localInstallLists'), 'scripts/server/install-package-lists %s' %pars)
    
    def createListsView(self):
        self.model = gtk.ListStore(bool,str, str, str)
        self.model.append([True, 'Κοινή λίστα για όλες τις βαθμίδες', 'gtk-edit', 
        '/etc/sch-scripts/koina.list'])
        self.model.append([False, 'Λίστα προγραμμάτων για το Δημοτικό', 'gtk-edit', 
        '/etc/sch-scripts/dimotiko.list'])
        self.model.append([False, 'Λίστα προγραμμάτων για το Γυμνάσιο', 'gtk-edit', 
        '/etc/sch-scripts/gymnasio.list'])
        self.model.append([False, 'Λίστα προγραμμάτων για το Λύκειο', 'gtk-edit', 
        '/etc/sch-scripts/lykeio.list'])
        
        tree = self.get('listsView')
        tree.set_model(self.model)
        
        col = gtk.TreeViewColumn()
        editcol = gtk.TreeViewColumn()
        
        tree.connect('row-activated', self.activate, editcol)
        tree.append_column(editcol)
        tree.append_column(col)
        
        celltog = gtk.CellRendererToggle()
        col.pack_start(celltog, False)
        celltog.connect('toggled', self.toggled)
        
        celltxt = gtk.CellRendererText()
        col.pack_start(celltxt)
        
        cellbtn = gtk.CellRendererPixbuf()
        editcol.pack_start(cellbtn, False)

        col.set_attributes(celltog, active=0)
        col.set_attributes(celltxt, text=1)
        editcol.set_attributes(cellbtn, stock_id=2)
    
    def openLocalListsDialog(self, widget):
        dlg=self.get('localLists')
        
        dlg.set_title("Τοπική εγκατάσταση λιστών προγραμμάτων")
        re=dlg.run()
        if re == 1:
            lists = [l[3] for l in self.model if l[0] == True]
            self.installListsOnServer(lists)
            
        dlg.hide()
    
    def toggled(self, widget, row):
        if row != '0': # So koina.list will be always selected
            self.model[row][0] = not self.model[row][0]
    
    def activate(self, widget, path, col, editcol):
        if col is editcol:
            path = self.model[path[0]][3]
            src = os.path.join('/usr/share/doc/sch-scripts/examples/', 
                                                os.path.split(path)[1])
            subprocess.Popen(['scripts/common/sch-install-example',
                                      src, path, str(False), str(True)])
    
    def runas_sudo_user(self, cmd):
        subprocess.Popen(['scripts/server/runas-sudo-user', '-dbus', self.dbus_session_bus_address ] + cmd)

    def openLink(self, link):
        subprocess.Popen(["xdg-open", link])
        
    def openHelpLink(self, widget):
        self.openLink("http://wiki.ubuntu-gr.org/sch-scripts")
        
    def openBugsLink(self, widget):
        self.openLink("https://bugs.launchpad.net/sch-scripts")
        
    def openQuestionsLink(self, widget):
        self.openLink("https://answers.launchpad.net/sch-scripts")
    
    def openForumLink(self, widget):
        self.openLink("http://users.sch.gr/alkisg/tosteki/index.php?board=67.0")

    def openIRCLink(self, widget):
        user = os.getenv("SUDO_USER")
        if user == None:
            user = "teacher."
        self.openLink("http://webchat.freenode.net/?nick=" + user + "&channels=linux.sch.gr&prompt=1")

    def remoteSupport(self, widget):
        dlg = self.get('remote_assistance_dialog')
        reply = dlg.run()
        if reply == 1:
            ip = self.get('rem_assist_ip').get_text()
            self.runas_sudo_user(['/usr/share/sch-scripts/scripts/server/sch-remote-assistance', ip.strip()])
        dlg.hide()
    
    def iconsSizeScaleChanged(self, widget):
        adj = self.get('iconsSizeAdjustment')
        self.scrWidth = int(adj.get_value())
        self.scrHeight = int(3*self.scrWidth/4) # So we keep the 4:3 aspect ratio
        self.getAllScreenshots()
        #self.get('clientsview').set_item_width(self.scrWidth)
        # Now the next screenshot will be in the above size
    
    def scrIncreaseSize(self, widget):
        # Increase the size of screenshots by 2 pixels in width
        adj = self.get('iconsSizeAdjustment')
        adj.set_value(adj.get_value() + 2)
        
    def scrDecreaseSize(self, widget):
        # Decrease the size of screenshots by 2 pixels in width
        adj = self.get('iconsSizeAdjustment')
        adj.set_value(adj.get_value() - 2)
        
    
    def showGroups(self, widget):
        self.sysgroups = self.get('sysgroups').get_active()
        self.emptygroups = self.get('emptygroups').get_active()
        self.gfilter.refilter()
        
    
    def contextMenuPopup(self, widget, event):
        clicked = widget.get_path_at_pos(int(event.x),int(event.y))
        
        if event.button == 3:
            if widget is self.cview:
                selection = widget
                selected = widget.get_selected_items()
            else:
                selection = widget.get_selection()
                selected = selection.get_selected_rows()[1]
                if clicked:
                    clicked = clicked[0]
            
            if clicked:
                if not clicked in selected:
                    selection.unselect_all()
                    selection.select_path(clicked)
            else:
                selection.unselect_all()
            
            if widget is self.cview:
                menu = self.get('clients').get_submenu()
            elif widget is self.utree:
                menu = self.get('users').get_submenu()
            elif widget is self.gtree:
                menu = self.get('groups').get_submenu()
            
            menu.popup(None,None,None,event.button,event.time)
            menu.show_all()
            return True
            
    def setPropertiesSensitivity(self, widget=None):
        selected = self.getSelectedClients()
        sensitive = False
        if len(selected) == 1:
            sensitive = True
        self.get('miClientProperties').set_sensitive(sensitive)
        self.get('barclientproperties').set_sensitive(sensitive)
        
    def selectUsersFromGroup(self, widget):
        selected = self.getSelectedGroups()
        users_selection = self.utree.get_selection()
        users_selection.unselect_all()
        users = []
        for row in selected:
            for member in row[2]:
                if not member in users: # Don't append the same user many times
                    users.append(member)
        
        for i in self.ustore:
            if i[0] in users:
                path = self.ufilter.convert_child_path_to_path(i.path)
                users_selection.select_path(path)
    
    def addUsersToGroup(self, widget):
        users = self.getSelectedUsers()
        groups = self.getSelectedGroups()
        for i in range(len(groups)):
            groups[i] = groups[i][0]
        groupstr = ','.join(groups)
        if groupstr == '':
            return
        for user in users:
            #Run the command and wait to finish before calling refresh()
            subprocess.call(['usermod', '-a', '-G', groupstr, user[0]])
        self.refresh()
        
    def removeUsersFromGroup(self, widget):
        users = self.getSelectedUsers()
        groups = self.getSelectedGroups()
        for i in range(len(groups)):
            groups[i] = groups[i][0]
        for user in users:
            remaining = user[5][:]
            for group in user[5]:
                if group in groups:
                    remaining.remove(group)
            remaining = ','.join(remaining)
            #Run the command and wait to finish before calling refresh()
            subprocess.call(['usermod', '-G', remaining, user[0]])
        self.refresh()
    
    def clientProperties(self, widget):
        selected = self.getSelectedClients()
        dlg = self.get('infodlg')
        execute = self.daemon.command
        set = lambda wdg, txt: self.get(wdg).set_text(txt.strip())
        
        for client in selected:
            set('client_ram', '')
            set('client_cpu', '')
            set('client_vga', '')
            if client[C_SYSTEM_HANDLE]:
                C_HANDLE = C_SYSTEM_HANDLE
            else:
                C_HANDLE = C_SESSION_HANDLE
            if client[C_HANDLE]:
                execute(client[C_HANDLE], 'echo $RAM').addCallback(lambda r: set('client_ram', r.strip()+' MB'))
                execute(client[C_HANDLE], 'echo $CPU').addCallback(lambda r: set('client_cpu', r))
                execute(client[C_HANDLE], 'echo $VGA').addCallback(lambda r: set('client_vga', r))
            set('hostname_entry', client[C_HOSTNAME])
            set('client_mac', client[C_MAC])
            set('client_ip', client[C_HANDLE].split(':')[0])
            set('client_type', client[C_TYPE])
            set('client_online_user', client[C_USER])
            dlg.set_title('Ιδιότητες του ' + client[C_HOSTNAME])
        dlg.run()
        dlg.hide()
    
    def editClientName(self, widget):
        dlg = self.get('entrydialog')
        entry = self.get('newhostname')
        
        if widget is self.get('change_name'):
            hostname = self.get('client_name').get_text()
            mac = self.get('client_mac').get_text()
            entry.set_text(hostname)
        
        resp = dlg.run()
        dlg.hide()
        
        if resp == 1:
            #self.changeHostname(mac, entry.get_text())
            self.refresh()
            print "No changes made, not yet implemented"
            
    def execOnClients(self, command, clients=[], reply=None, root=False, handles=[], warning=''):
        '''reply should be a method in which the result will be sent'''
        if len(self.cstore) == 0:
            # print 'No clients'
            return False
        if (clients !=[] or handles !=[]) and warning !='':
            if self.warnDlgPopup(warning) == False:
                return
        if clients == [] and handles != []:
            for handle in handles:
                cmd = self.daemon.command(handle, unicode(command))
                cmd.addErrback(self.printErrors)
                if reply:
                    cmd.addCallback(lambda re, h=handle: reply(h, re))
        for client in clients:
            if (root == "auto" or not root) and client[C_SESSION_HANDLE] != '':
                handle = client[C_SESSION_HANDLE]
            elif root and client[C_SYSTEM_HANDLE] != '':
                handle = client[C_SYSTEM_HANDLE]
            else:
                continue
            cmd = self.daemon.command(handle, unicode(command))
            cmd.addErrback(self.printErrors)
            if reply:
                cmd.addCallback(lambda re, h=handle: reply(h, re))

    def execOnSelectedClients(self, command, reply=None, root=False, warn=''):
        clients = self.getSelectedClients()
        # If there is no selected client, send the command to all
        if len(clients) == 0:
            clients = self.cstore
        else:
            # Only show the warning when no clients are selected
            warn=''
        self.execOnClients(command, clients, reply, root, warning=warn)
    
    def warnDlgPopup(self, warning):
        dlg = self.get('executionwarning')
        dlg.set_property('text', warning)
        resp = dlg.run()
        dlg.set_property('text', '')
        dlg.hide()
        # -8 is the response id of the "Yes" button (-9 for the "No")
        if resp == -8:
            return True
        else:
            return False
    
    def printErrors(self, error):
        print 'ErrorCallback:', error
        return
    
    def execDialog(self, widget):
        cmd = startExecuteCmdDlg()
        # If Cancel or Close were clicked
        if cmd == 0 or cmd == -4:
            return
        as_root = False
        if cmd[:5] == 'sudo ':
            as_root = True
            cmd = cmd[4:]
        self.execOnSelectedClients('./sch-exec ' + cmd, root=as_root)

    def wake_on_lan(self, widget):
        for client in self.getSelectedClients():
            # Make sure that only offline computers will be sent to wol
            if client[C_SESSION_HANDLE] == '' and client[C_SYSTEM_HANDLE] == '':
                wol.wake_on_lan(client[C_MAC])

    def poweroff(self, widget):
        self.execOnSelectedClients('./sch-poweroff', root="auto", warn='Θέλετε σίγουρα να τερματίσετε όλους τους υπολογιστές;')

    def reboot(self, widget):
        # FIXME: (Not) waiting on purpose to cause some delay to avoid 
        # any power strain.
        self.execOnSelectedClients('./sch-reboot', root="auto", warn='Θέλετε σίγουρα να επανεκκινήσετε όλους τους υπολογιστές;')

    def logout(self, widget):
        self.execOnSelectedClients('./sch-logout', warn='Θέλετε σίγουρα να αποσυνδέσετε όλους τους χρήστες;')

    def monitorStudent(self, widget, path=None, view_column=None):
        # Tell vinagre to listen for incoming connections
        self.runas_sudo_user(['gconftool-2', '--set', '/apps/vinagre/always_enable_listening',
            '--type', 'boolean', '1'])

        # Open vinagre
        self.runas_sudo_user(['vinagre'])

        # And, tell the clients to connect to the server
        self.execOnSelectedClients('./sch-exec x11vnc -noshm -connect_or_exit $SERVER')

    def broadcastTeacher(self, widget):
        if not self.vncserver_running:
            self.vncserver_running = True
            # TODO: switch to using -autoport
            self.runas_sudo_user(['x11vnc', '-nopw', '-quiet', '-viewonly', '-shared', '-forever',
                '-nolookup', '-rfbport', '5903', '-allow', '127.,192.168.,10.,169.254.' ])
        self.execOnSelectedClients("""killall gnome-screensaver 2>/dev/null""")
        self.execOnSelectedClients("""test -n "$SCH_VNCVIEWER_PID" && kill $SCH_VNCVIEWER_PID
export DISPLAY=$(cat /proc/$(pidof ldm)/environ | tr '\\0' '\\n' | awk -F'=' '/DISPLAY=/ { print $2 }')
export XAUTHORITY=$(cat /proc/$(pidof ldm)/environ | tr '\\0' '\\n' | awk -F'=' '/XAUTHORITY=/ { print $2 }')
xset dpms force on
sleep 0.$((`hexdump -e '"%d"' -n 2 /dev/urandom` % 50 + 50))
SCH_VNCVIEWER_PID=$( ./sch-exec xvnc4viewer -Shared -ViewOnly -FullScreen -UseLocalCursor=0 -MenuKey F13 server:3)""", root=True)

    def stopTransmissions(self, widget):
        # The clients are usually automatically killed when the server is killed
        # Unfortunately, not always, so try to kill them anyway
        self.execOnClients('test -n "$SCH_VNCVIEWER_PID" && kill $SCH_VNCVIEWER_PID', self.cstore, None, True)
        # TODO: remember x11vnc pid and kill it
        self.runas_sudo_user(['killall', 'x11vnc'])
        self.runas_sudo_user(['killall', '-9', 'x11vnc'])
        self.vncserver_running = False

    def lockScreen(self, widget):
        self.execOnSelectedClients('test -n "$SCH_LOCK_SCREEN_PID" && kill "$SCH_LOCK_SCREEN_PID"; SCH_LOCK_SCREEN_PID=$(./sch-exec ./sch-lock-screen)')
        #self.execOnSelectedClients('SCH_LOCK_SCREEN_PID=$(./sch-exec ./sch-lock-screen)')
    
    def unlockScreen(self, widget):
        self.execOnSelectedClients('''test -n "$SCH_LOCK_SCREEN_PID" && kill "$SCH_LOCK_SCREEN_PID"
unset SCH_LOCK_SCREEN_PID''')
    
    def soundOff(self, widget):
        self.execOnSelectedClients('./sch-exec amixer -c 0 -q sset Master mute', root=True)
    
    def soundOn(self, widget):
        self.execOnSelectedClients('./sch-exec amixer -c 0 -q sset Master unmute', root=True)
    
    def killUserProcesses(self, widget):
        users = self.getSelectedUsers()
        if self.warnDlgPopup('Θέλετε σίγουρα να τερματίσετε όλες τις διεργασίες των επιλεγμένων χρηστών;') == False:
            return
        for user in users:
            command = "killall -u %s && sleep 2 && killall -9 -u %s" %(user[0], user[0])
            subprocess.Popen(['/bin/sh', '-c', command])
        gobject.timeout_add(3000, self.loadUsers)
        
    def execInTerminal(self, widget, command):
        name = widget.get_child().get_text()
        subprocess.Popen([ 'x-terminal-emulator', '-e', 'sh', '-c', command
            + ' && read -p "Sch-script \'%s\' finished succesfully. Press [Enter] to close this window." dummy' %name
            + ' || read -p "Sch-script \'%s\' finished with errors. Press [Enter] to close this window." dummy' %name]
            )

    def installExample(self, src, dst, expand=False, open=True):
        subprocess.Popen(['scripts/common/sch-install-example',
            src, dst, str(expand), str(open)])

    def edit_lts_conf(self, widget):
        self.installExample('../doc/sch-scripts/examples/lts.conf', '/var/lib/tftpboot/ltsp/i386/lts.conf')

    def edit_ltsp_build_client_conf(self, widget):
        self.installExample('../doc/sch-scripts/examples/ltsp-build-client.conf', '/etc/ltsp/ltsp-build-client.conf')

    def edit_dnsmasq_conf(self, widget):
        self.installExample('../doc/sch-scripts/examples/dnsmasq.conf', '/etc/dnsmasq.d/ltsp.conf', True, True)

    def edit_koina_list(self, widget):
        self.installExample('../doc/sch-scripts/examples/koina.list', '/etc/sch-scripts/koina.list')

    def edit_dimotiko_list(self, widget):
        self.installExample('../doc/sch-scripts/examples/dimotiko.list', '/etc/sch-scripts/dimotiko.list')

    def edit_gymnasio_list(self, widget):
        self.installExample('../doc/sch-scripts/examples/gymnasio.list', '/etc/sch-scripts/gymnasio.list')

    def edit_lykeio_list(self, widget):
        self.installExample('../doc/sch-scripts/examples/lykeio.list', '/etc/sch-scripts/lykeio.list')

    def sch_ltsp_build_client(self, widget):
        subprocess.Popen(['./ltspbuildclientconf_gen.py'])
    
    def sch_ltsp_enter_chroot(self, widget):
        self.execInTerminal(widget, 'scripts/server/sch-ltsp-enter-chroot')

    def sch_ltsp_update_image(self, widget):
        self.execInTerminal(widget, 'scripts/server/sch-ltsp-update-image')

    def sch_ltsp_upgrade_chroot(self, widget):
        self.execInTerminal(widget, 'scripts/server/sch-ltsp-upgrade-chroot')

    
