###########################################################################
# Contains miscellaneous utility functions for chroot-handling sch-scripts.
#
# Copyright (C) 2010 Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL'.
###########################################################################

# Prevent multiple inclusions
test -z "$INCLUDED_CHROOT" || return 0

enter_chroot() {
    chroot "$ROOT" mount -t proc proc /proc || die "Not a valid chroot: $ROOT"
    mount --bind /var/cache/apt/archives "$ROOT/var/cache/apt/archives"
    cp /etc/resolv.conf "$ROOT/etc/"
    export LTSP_HANDLE_DAEMONS=false
}

exit_chroot() {
    unset LTSP_HANDLE_DAEMONS
    umount "$ROOT/var/cache/apt/archives"    
    # binfmt_misc might need to be unmounted manually, see LP #534211
    if [ $(grep ^binfmt_misc "$ROOT/proc/mounts" 2>/dev/null | wc -l) -gt 1 ]; then
        umount "$ROOT/proc/sys/fs/binfmt_misc" || true
    fi
    if ! umount "$ROOT/proc"; then
        echo_bold "$ROOT/proc is in use, forcing unmount!"
        umount -l "$ROOT/proc"
    fi
}

# Prevent multiple inclusions
INCLUDED_CHROOT=true
